=== modified file 'NEWS'
--- NEWS	2012-02-02 07:42:24 +0000
+++ NEWS	2012-02-03 01:29:46 +0000
@@ -55,6 +55,9 @@
     - Make tz calculations consistent and use UTC in the UI everywhere we show
       a precise timestamp. (Robert Collins, #594591)
 
+    - Avoid crashing when viewing or annotating a non-existent file.
+      (William Grant, #728209)
+
 
 1.18.1 [24Mar2011]
 ------------------

=== modified file 'loggerhead/controllers/view_ui.py'
--- loggerhead/controllers/view_ui.py	2011-09-08 00:33:28 +0000
+++ loggerhead/controllers/view_ui.py	2012-02-03 01:25:58 +0000
@@ -24,7 +24,12 @@
 import bzrlib.textfile
 import bzrlib.osutils
 
-from paste.httpexceptions import HTTPBadRequest, HTTPServerError, HTTPMovedPermanently
+from paste.httpexceptions import (
+    HTTPBadRequest,
+    HTTPMovedPermanently,
+    HTTPNotFound,
+    HTTPServerError,
+    )
 
 from loggerhead.controllers import TemplatedBranchView
 try:
@@ -119,7 +124,12 @@
             raise HTTPServerError('Could not fetch changes')
         branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
 
-        if inv[file_id].kind == "directory":
+        try:
+            file = inv[file_id]
+        except bzrlib.errors.NoSuchId:
+            raise HTTPNotFound()
+
+        if file.kind == "directory":
             raise HTTPMovedPermanently(self._branch.context_url(['/files', revno_url, path]))
 
         return {

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2011-11-25 01:41:07 +0000
+++ loggerhead/tests/test_controllers.py	2012-02-03 01:25:58 +0000
@@ -22,7 +22,10 @@
 from paste.fixture import (
     AppError,
     )
-from paste.httpexceptions import HTTPServerError
+from paste.httpexceptions import (
+    HTTPNotFound,
+    HTTPServerError,
+    )
 
 from testtools.matchers import (
     Matcher,
@@ -202,6 +205,13 @@
         annotated = annotate_info['annotated']
         self.assertEqual(0, len(annotated))
 
+    def test_annotate_nonexistent_file(self):
+        history = [('rev1', '' , '.')]
+        ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
+        ann_ui.args = ['rev1']
+        self.assertRaises(
+            HTTPNotFound, ann_ui.get_values, 'not-filename', {}, {})
+
 
 class TestFileDiffUI(BasicTests):
 

